/*
** Module   : tty.keyboard.js
** Abstract : keystrokes management.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20140110 First version.
*/

tty.keyboard = (function(socket)
{
   /* exports object */
   var my = {}; 

   /* normal key mapping */
   var xlate = {};

   /* ALT key mapping */
   var xlateAlt = {};
   
   /* track escape status */
   var escape = false;
            
   /* send keyEvent */
   var sendKey = function(key, evt)
   {
      var data = {};
      data.c=0;
      data.k=key;
      socket.send(JSON.stringify(data));
      evt.preventDefault();
   }
         
   /* initialize module */
   my.init = function()
   {      
      require(["dojo/keys", "dojo/dom", "dojo/on", "dojo/domReady!"], function(keys, dom, on)
      {  
         /* keypress */
         on(document, "keypress", function(evt)
         {
            var key = evt.charCode;
            if (key != 0)
            {
               sendKey(key, evt);
            }
         });
                    
         /* keydown */
         on(document, "keydown", function(evt)
         {
            var key = evt.keyCode;

            if (key == keys.ESCAPE)
            {
               escape = !escape;
               return;
            }

            if (key == keys.CTRL || key == keys.ALT)
            {
               return;
            }
            
            var p4code = key;
            
            // Normal
            if(!evt.altKey && !evt.ctrlKey && !escape)
            {
               /* normal map */
               p4code = xlate[key];
               
               if(p4code !== undefined)
               {
                  if(p4code == keys.TAB && evt.shiftKey)
                  {
                     p4code = 509;
                  }                           
                  sendKey(p4code, evt);
               }
               return;
            }
            
            // Handle CTRL
            if(evt.ctrlKey)
            {
               if(key == 50)  //'2'
               {
                  p4code = 0;
               }
               else if(key == 54) //'6'
               {
                  p4code = 30;
               }
               else if(key == 173) //'-'
               {
                  p4code = 31;
               }
               else
               {
                  if(key < 127)
                     p4code = key & 0xffbf;
               }
            }
                    
            var p4codeAlt = p4code;
               
            // Handle ALT or Esc
            if(evt.altKey || escape)
            {
               p4codeAlt = xlateAlt[p4code];
               
               if(p4codeAlt === undefined)
               {
                  p4codeAlt = (escape && evt.altKey) ? -1 : p4code | 1024;
               }
               else
               {
                  if (!escape && evt.altKey && p4codeAlt == keys.DELETE)
                  {
                     // special case for ALT+DELETE key combo
                     p4codeAlt = 512;
                  }            
               }
               escape = false;
            }            

            if(p4codeAlt != -1)
            {
               sendKey(p4codeAlt, evt);
            }
            
         });//on key
               
         /* p4g map */
         var map4gkeys = function()
         {
            // Normal keys
            xlate[keys.BACKSPACE]= 8;
            xlate[keys.TAB]= 9;
            xlate[keys.ENTER]= 13;
            xlate[keys.UP_ARROW]= 501;
            xlate[keys.DOWN_ARROW]=502;
            xlate[keys.RIGHT_ARROW]=503;
            xlate[keys.LEFT_ARROW]=504;
            xlate[keys.HOME]= 505;
            xlate[keys.END]= 506;
            xlate[keys.PAGE_UP]= 507;
            xlate[keys.PAGE_DOWN]= 508;
            xlate[keys.INSERT]=510;
            xlate[keys.DELETE]=127;     
            xlate[keys.F1]= 301;
            xlate[keys.F2]= 302;
            xlate[keys.F3]= 303;
            xlate[keys.F4]= 304;
            xlate[keys.F5]= 305;
            xlate[keys.F6]= 306;
            xlate[keys.F7]= 307;
            xlate[keys.F8]= 308;
            xlate[keys.F9]= 309;
            xlate[keys.F10]= 310;
            xlate[keys.F11]= 311;
            xlate[keys.F12]= 312;
            
            // ALT-keys
            xlateAlt[keys.UP_ARROW] = 559;      
            xlateAlt[keys.DOWN_ARROW] = 518;
            xlateAlt[keys.RIGHT_ARROW] = 553;
            xlateAlt[keys.LEFT_ARROW] = 538;
            xlateAlt[keys.PAGE_UP] = 548;
            xlateAlt[keys.PAGE_DOWN] = 543;
            xlateAlt[keys.INSERT] = 533;
            xlateAlt[keys.DELETE]=127;     
            xlateAlt[keys.HOME] = 528;
            xlateAlt[keys.END] = 523;
            xlateAlt[keys.F1] = 313;
            xlateAlt[keys.F2] = 314;
            xlateAlt[keys.F3] = 315;
            xlateAlt[keys.F4] = 316;
            xlateAlt[keys.F5] = 317;
            xlateAlt[keys.F6] = 318;
            xlateAlt[keys.F7] = 319;
            xlateAlt[keys.F8] = 320;
            xlateAlt[keys.F9] = 321;
            xlateAlt[keys.F10] = 322;
            xlateAlt[keys.F11] = 323;
            xlateAlt[keys.F12] = 324;
         };
         
         /* map keys */
         map4gkeys();      
                  
      });//require
                  
   };//init
         
   /* exports */
   return my;    
}(tty.socket));
